/*
 *		XPfunctionsgeometry.h
 *
 *	(C) 2003 ramjac Software/Patrick Enoch
 *
 *	geometry generating functions
 *
 *	!! they must check xpression_data->generateGeometry if they are allowed to generate !!
 */
#ifndef __XP_FUNCTIONSGEOMETRY_
#define __XP_FUNCTIONSGEOMETRY_

/*
 *	mesh3d ( vec(minx, maxx, xdivisions), vec(miny, maxy, ydivisions), vec( [formula(x,y)] )
 *
 *	generates a 3d-mesh
 *
 *	define mesh3d-function as a macro, because arg3 is a formula like "vec(x,y,x*y)"
 */
void mesh3d_addmesh(vec a, vec b, vec* points);
#define mesh3d(arg1,arg2,arg3)\
{												\
	if (xpression_data->generateGeometry)		\
	{											\
		vec _a=arg1;							\
		vec _b=arg2;							\
		long _xarrdim=(long)_a.Z;				\
		long _yarrdim=(long)_b.Z;				\
		vec _varr[_xarrdim+1][_yarrdim+1];		\
		double _xdim=(_a.Y-_a.X)/_a.Z;			\
		double _ydim=(_b.Y-_b.X)/_b.Z;			\
		for (long _x=0; _x<=_xarrdim; _x++)		\
		{										\
			double x=_a.X+_xdim*_x;				\
			for (long _y=0; _y<=_yarrdim; _y++)	\
			{									\
				double y=_b.X+_ydim*_y;			\
				_varr[_x][_y]=arg3;				\
			};									\
		};										\
		mesh3d_addmesh(_a,_b,&_varr[0][0]);		\
	};											\
};

/*
 *	mesh ( vec(minx, maxx, xdivisions), vec(miny, maxy, ydivisions), [heightformula(x,y)] )
 *
 *	generates a landscape type of mesh
 *
 *	define mesh-function as a macro, because arg3 is a formula like "x*y"
 */
#define mesh(arg1,arg2,arg3)					\
{												\
	if (xpression_data->generateGeometry)		\
	{											\
		vec _a=arg1;							\
		vec _b=arg2;							\
		long _xarrdim=(long)_a.Z;				\
		long _yarrdim=(long)_b.Z;				\
		vec _varr[_xarrdim+1][_yarrdim+1];		\
		double _xdim=(_a.Y-_a.X)/_a.Z;			\
		double _ydim=(_b.Y-_b.X)/_b.Z;			\
		for (long _x=0; _x<=_xarrdim; _x++)		\
		{										\
			double x=_a.X+_xdim*_x;				\
			for (long _y=0; _y<=_yarrdim; _y++)	\
			{									\
				double y=_b.X+_ydim*_y;			\
				_varr[_x][_y]=vec(x,arg3,y);	\
			};									\
		};										\
		mesh3d_addmesh(_a,_b,&_varr[0][0]);		\
	};											\
};

/*
 *	face(3,a,b,c) and face(4,a,b,c,d)
 */
void face(int, vec a, vec b, vec c);

/*
 *	line(vec,vec,vec,vec ...)
 */
void line(vec a, vec b);
void line(vec a, vec b, vec c);
void line(vec a, vec b, vec c, vec d);
inline void line(vec a, vec b, vec c, vec d, vec e)
{
	line(a,b,c,d);
	line(d,e);
}
inline void line(vec a, vec b, vec c, vec d, vec e, vec f)
{
	line(a,b,c,d);
	line(d,e,f);
}

#define create_cross(...)
#define create_box(...)
#define create_joint(...)
#define create_bone(...)
#define create_camera(...)
#define create_cameralight(...)
#define create_parallellight(...)
#define create_radiallight(...)
#define create_ambientlight(...)
#define create_spotlight(...)
#define create_tubelight(...)
#define create_radialdomeilluminator(...)
#define create_radialboxilluminator(...)
#define create_smoker(...)


#endif
