/*
 *		XPfunctionsvector.h
 *
 *	(C) 2003 ramjac Software/Patrick Enoch
 *
 *	math on vectors

	symbol::addSystem(symbol_STRUCTFUNC, 	"childpos", 	(void*)_childpos);
   	symbol::addSystem(symbol_FUNCTION,		"coord",		(void*)_coord);
   	symbol::addSystem(symbol_FUNCTION,		"hermite",		(void*)_hermite);
	symbol::addSystem(symbol_FUNCTION, 		"max", 			(void*)_max);
	symbol::addSystem(symbol_FUNCTION,		"min", 			(void*)_min);

 */
#ifndef __XP_FUNCTIONSVECTOR_
#define __XP_FUNCTIONSVECTOR_

/*
 *	some global "constants" that might be defined later than this include
 */
extern double frame;
extern double fps;

inline double length(vec a)	{	return a.length();	}
inline double length(vec4 a){	return a.length();	}
inline double mag(vec a)	{	return length(a);	}
inline double mag(vec4 a)	{	return length(a);	}

inline double distance(vec a, vec b)	{	return length(a-b);	}
inline double distance(vec4 a, vec4 b)	{	return length(a-b);	}

/*
 *	difference vec to frame-1
 */
inline vec vdelta(indexed_vec a)	{	return (a - a[frame-1]);	}
inline vec4 vdelta(indexed_vec4 a)	{	return (a - a[frame-1]);	}
inline double delta(indexed_double a){	return (a - a[frame-1]);	}

/*
 *	difference vec to frame-1 multiplied by fps
 */
inline vec vspeed(indexed_vec a)	{	return vdelta(a) * fps;	}
inline vec4 vspeed(indexed_vec4 a)	{	return vdelta(a) * fps;	}
inline double speed(indexed_double a){	return delta(a) * fps;	}

/*
 *	av() average vec/vec4/doubles
 */
inline double av(double a, double b) { return (a+b)/2; }
inline double av(double a, double b, double c) { return (a+b+c)/3; }
inline double av(double a, double b, double c, double d) { return (a+b+c+d)/4; }
inline double av(double a, double b, double c, double d, double e) { return (a+b+c+d+e)/5; }
inline double av(double a, double b, double c, double d, double e, double f) { return (a+b+c+d+e+f)/6; }

inline vec av(vec a, vec b) { return (a+b)/2; }
inline vec av(vec a, vec b, vec c) { return (a+b+c)/3; }
inline vec av(vec a, vec b, vec c, vec d) { return (a+b+c+d)/4; }
inline vec av(vec a, vec b, vec c, vec d, vec e) { return (a+b+c+d+e)/5; }
inline vec av(vec a, vec b, vec c, vec d, vec e, vec f) { return (a+b+c+d+e+f)/6; }

inline vec4 av(vec4 a, vec4 b) { return (a+b)/2; }
inline vec4 av(vec4 a, vec4 b, vec4 c) { return (a+b+c)/3; }
inline vec4 av(vec4 a, vec4 b, vec4 c, vec4 d) { return (a+b+c+d)/4; }
inline vec4 av(vec4 a, vec4 b, vec4 c, vec4 d, vec4 e) { return (a+b+c+d+e)/5; }
inline vec4 av(vec4 a, vec4 b, vec4 c, vec4 d, vec4 e, vec4 f) { return (a+b+c+d+e+f)/6; }

/*
 *	sqr() can square double/vec/vec4
 */
inline double sqr(double a)	 	{ return a*a; }
inline double sqr(vec a) 		{ return a*a; }
inline double sqr(vec4 a)		{ return a*a; }

/*
 *	angle between two vectors
 */
inline double angle(vec a,vec b)
{
	double len = length(a)*length(b);
	if (len==0)
		return 0;
	return acos( (a*b) / len );
}

/*
 *	cross product (only 3d)
 */
inline vec cross(vec a,vec b)
{
	return vec( a.Y*b.Z - a.Z*b.Y , a.Z*b.X - a.X*b.Z , a.X*b.Y - a.Y*b.X );
}

/*
 *	spat product (only 3d)
 */
inline double spat(vec x,vec y,vec z)
{
	return cross(x,y) * z;
}

/*
 *	unit vector of same direction
 */
inline vec unit(vec a)
{
	double l=length(a);
	if (l != 0)
	{
		return a / l;
	}
	return a;
}
inline vec4 unit(vec4 a)
{
	double l=length(a);
	if (l != 0)
	{
		return a / l;
	}
	return a;
}

/*
 *	alternative functions to get member / component ( = XYZ)
 */
inline double xcomp(vec a)	{	return a.X;	}
inline double ycomp(vec a)	{	return a.Y;	}
inline double zcomp(vec a)	{	return a.Z;	}

inline vec random( vec a )
{
	return vec( a.X*XPinternal_random()/XPinternal_RAND_MAX ,
				a.Y*XPinternal_random()/XPinternal_RAND_MAX ,
				a.Z*XPinternal_random()/XPinternal_RAND_MAX );
}

vec lookat( vec a, vec b );

vec hermite(vec arg1, vec arg2, vec arg3, vec arg4, double a)
{
	vec B=arg1+arg3;
	vec C=arg2+arg4;
						 
	double b=1-a;

	vec res = b*b*b*arg1 + 3*B*b*b*a + 3*C*b*a*a + arg2*a*a*a;
	
	return res;
}

#endif
