/*
 *		XPindexed_types.h
 *
 *	(C) 2003-2004 Patrick Enoch/ramjac Software
 *
 *	they need to be initialized by XP to their proper IDs (load_value) first
 *	(that part of the expression is generated by XP)
 *
 *	after that they behave like double/vec/vec4 except that they can be indexed[]
 *	to get their value at a certain frame
 */
#ifndef __XP_INDEXEDTYPES_
#define __XP_INDEXEDTYPES_

/*
 *	some global "constants" that might be defined later than this include
 */
extern CalculatorDataPtr xpression_data;

/*
 *	prototype for an indexable value
 *
 *	it knows a structure to identify itself to XP for indexing by frame
 */
class XPindexed_value
{
public:
inline void			base_load_value(ChannelPtr a)
					{
						// backup old values
						orig_values[0] = a->value[0];
						orig_values[1] = a->value[1];
						orig_values[2] = a->value[2];
						orig_values[3] = a->value[3];
						// we will work on the values directly!
						indexed_infos = a;
					}

inline void			base_save_value(ChannelPtr a)
					{
						// compare to old values and set "changed" attribute accordingly
						if ((orig_values[0] != a->value[0]) ||
							(orig_values[1] != a->value[1]) ||
							(orig_values[2] != a->value[2]) ||
							(orig_values[3] != a->value[3])   )
						{
							a->changed=true;
						}
					}

inline void			_operator_index(const long frame, double value[])
					{
						#ifdef __MACCARBON__
							// CFM-call: *void = (addr,toc=r2)
							// jump to *(*void)
							// pass r2 as first argument and let the function fix it itself
							(*(void(*)(unsigned long,PluginDataPtr,ChannelPtr,long,double[]))(*xpression_data->IndexedVariableDLL))
								(((unsigned long*)xpression_data->IndexedVariableDLL)[1],xpression_data->theData, indexed_infos, frame, value);
						#else
							(*(void(*)(PluginDataPtr,ChannelPtr,long,double[]))(xpression_data->IndexedVariableDLL))
								(xpression_data->theData, indexed_infos, frame, value);
						#endif
					}

	ChannelPtr	 	indexed_infos;

private:

	double			orig_values[4];
};

/*
 *	indexable double
 */
class indexed_double: public XPindexed_value
{
public:
inline void			load_value(ChannelPtr a)
					{
						base_load_value(a);
					}

inline void			save_value(ChannelPtr a)
					{
						base_save_value(a);
					}

inline indexed_double&	operator =(indexed_double& a)
					{
						indexed_infos->value[0] = a.indexed_infos->value[0];
						return *this;
					}

inline indexed_double&	operator +=(const double a)
					{
						indexed_infos->value[0] += a;
						return *this;
					}

inline indexed_double&	operator -=(const double a)
					{
						indexed_infos->value[0] -= a;
						return *this;
					}

inline indexed_double&	operator *=(const double a)
					{
						indexed_infos->value[0] *= a;
						return *this;
					}

inline indexed_double&	operator /=(const double a)
					{
						indexed_infos->value[0] /= a;
						return *this;
					}

inline				operator const double() const
					{
						return indexed_infos->value[0];
					}

inline indexed_double&	operator =(const double a)
					{
						indexed_infos->value[0] = a;
						return *this;
					}

inline double	operator [](const double frame)
					{
						double val=0;
						_operator_index((long)frame,&val);
						return val;
					}

/*
inline indexed_double&	operator =(const indexed_double a)
					{
						indexed_infos->value[0] = a.indexed_infos->value[0];
						return *this;
					}
*/
/*
					// override copy-constructor
inline indexed_double&	operator =(const indexed_double& a)
					{
						indexed_infos->value[0] = a.indexed_infos->value[0];
						return *this;
					}
*/

};

/*
 *	indexable n-th element of a vec/vec4 (defined below)
 *
 *	elementnum ranges from 0=X/R/H, 1=Y, 2=Z, 3=A
 */
class XPindexed_vec_element: public XPindexed_value
{
public:
inline void	load_value(ChannelPtr a, const long _elementnum)
			{
				base_load_value(a);
				elementnum=_elementnum;
			}

inline		operator const double()
			{
				return indexed_infos->value[elementnum];
			}

inline double	operator [](const double frame)
			{
				double val[4]={0,0,0,0};
				_operator_index((long)frame,val);
				return val[elementnum];
			}

inline 		XPindexed_vec_element& operator =(const double& a)
			{
				indexed_infos->value[elementnum] = a;
				return *this;
			}

			// copy assignment-operator (doesnt work with const XPindexed_vec_element&)
inline 		XPindexed_vec_element& operator =( XPindexed_vec_element& a)
			{
				indexed_infos->value[elementnum] = a;
				return *this;
			}

inline		XPindexed_vec_element& operator +=(const double a)
			{
				indexed_infos->value[elementnum] += a;
				return *this;
			}	

inline		XPindexed_vec_element& operator -=(const double a)
			{
				indexed_infos->value[elementnum] -= a;
				return *this;
			}	

inline		XPindexed_vec_element& operator *=(const double a)
			{
				indexed_infos->value[elementnum] *= a;
				return *this;
			}	

inline		XPindexed_vec_element& operator /=(const double a)
			{
				indexed_infos->value[elementnum] /= a;
				return *this;
			}	

private:

	long		elementnum;
};

/*
 *	indexable vec
 *
 *	can be used for XYZ, RGB and HSV
 */
class indexed_vec: public XPindexed_value
{
public:
	XPindexed_vec_element	X,R,H;
	XPindexed_vec_element	Y,G,S;
	XPindexed_vec_element	Z,B,V;

inline void	load_value(ChannelPtr a)
			{
				base_load_value(a);
				X.load_value(a,0);
				R.load_value(a,0);
				H.load_value(a,0);

				Y.load_value(a,1);
				G.load_value(a,1);
				S.load_value(a,1);

				Z.load_value(a,2);
				B.load_value(a,2);
				V.load_value(a,2);
			}

inline void	save_value(ChannelPtr a)
			{
				base_save_value(a);
			}
			
inline		indexed_vec& operator =( indexed_vec& a)
			{
				X=a.X;
				Y=a.Y;
				Z=a.Z;
				return *this;
			}			

inline		indexed_vec& operator +=( indexed_vec& a)
			{
				X+=a.X;
				Y+=a.Y;
				Z+=a.Z;
				return *this;
			}			

inline		indexed_vec& operator -=( indexed_vec& a)
			{
				X-=a.X;
				Y-=a.Y;
				Z-=a.Z;
				return *this;
			}			

inline		indexed_vec& operator *=( const double a)
			{
				X*=a;
				Y*=a;
				Z*=a;
				return *this;
			}

inline		operator const vec()
			{
				return vec(X,Y,Z);
			}

inline		indexed_vec& operator =(const vec a)
			{
				X=a.X;
				Y=a.Y;
				Z=a.Z;
				return *this;
			}

inline	const vec	operator [](const double frame)
			{
				double val[3]={0,0,0};
				_operator_index((long)frame,val);
				return vec(val[0],val[1],val[2]);
			}
};

/*
 *	indexable vec4
 *
 *	can be used for XYZA, RGBA and HSVA
 */
class indexed_vec4: public XPindexed_value
{
public:
	XPindexed_vec_element	X,R,H;
	XPindexed_vec_element	Y,G,S;
	XPindexed_vec_element	Z,B,V;

	XPindexed_vec_element	A;

inline void	load_value(ChannelPtr a)
			{
				base_load_value(a);
				X.load_value(a,0);
				R.load_value(a,0);
				H.load_value(a,0);

				Y.load_value(a,1);
				G.load_value(a,1);
				S.load_value(a,1);

				Z.load_value(a,2);
				B.load_value(a,2);
				V.load_value(a,2);

				A.load_value(a,3);
			}

inline void	save_value(ChannelPtr a)
			{
				base_save_value(a);
			}

inline		indexed_vec4& operator =( indexed_vec4& a)
			{
				X=a.X;
				Y=a.Y;
				Z=a.Z;
				A=a.A;
				return *this;
			}

inline		indexed_vec4& operator +=( indexed_vec4& a)
			{
				X+=a.X;
				Y+=a.Y;
				Z+=a.Z;
				A+=a.A;
				return *this;
			}

inline		indexed_vec4& operator -=( indexed_vec4& a)
			{
				X-=a.X;
				Y-=a.Y;
				Z-=a.Z;
				A-=a.A;
				return *this;
			}

inline		indexed_vec4& operator *=( const double a)
			{
				X*=a;
				Y*=a;
				Z*=a;
				A*=a;
				return *this;
			}

inline		operator const vec4()
			{
				return vec4(X,Y,Z,A);
			}

inline		indexed_vec4& operator =(const vec4 a)
			{
				X=a.X;
				Y=a.Y;
				Z=a.Z;
				A=a.A;
				return *this;
			}

inline vec4	operator [](double frame)
			{
				double val[4]={0,0,0,0};
				_operator_index((long)frame,val);
				return vec4(val[0],val[1],val[2],val[3]);
			}
};

#endif
